/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.db.expr;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

import net.morilib.db.delay.Delay;
import net.morilib.db.engine.SqlEngine;
import net.morilib.db.relations.RelationAggregate;
import net.morilib.db.relations.RelationTuple;
import net.morilib.db.schema.SqlSchema;

public class RelationUnaryOperate extends RelationExpression {

	//
	private RelationUnaryOperator op;
	private RelationExpression pa;

	public RelationUnaryOperate(RelationUnaryOperator o,
			RelationExpression a) {
		op = o;
		pa = a;
	}

	@Override
	public Object eval(SqlEngine v, SqlSchema f,
			RelationTuple tuple, RelationAggregate m,
			List<String> group,
			List<Object> h) throws IOException, SQLException {
		final Object o;

		o = pa.eval(v, f, tuple, m, group, h);
		if(o instanceof Delay) {
			return o;
		} else {
			return op.op(o);
		}
	}

	@Override
	public boolean isAggregate() throws SQLException {
		return pa.isAggregate();
	}

	@Override
	public Object init(SqlEngine v, SqlSchema f) throws SQLException {
		final Object o = pa.init(v, f);

		if(o instanceof Delay) {
			return new Delay() {

				@Override
				public Object force() throws SQLException {
					return op.op(((Delay)o).force());
				}

				@Override
				public void add(Object... args) throws SQLException {
					((Delay)o).add(args);
				}

			};
		} else {
			return op.op(o);
		}
	}

}
