package net.morilib.db.jdbc;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.Ref;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import net.morilib.db.misc.ErrorBundle;

public class NullResultSet extends UnupdatableResultSet {

	private static Logger logger = RelationsJDBCUtils.logger;

	List<String> columns;

	NullResultSet(String... names) {
		columns = Arrays.asList(names);
	}

	@Override
	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		return false;
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		throw new SQLException("not supported");
	}

	@Override
	public boolean next() throws SQLException {
		logger.finer("next");
		return false;
	}

	@Override
	public void close() throws SQLException {
		// ignore it
	}

	@Override
	public boolean wasNull() throws SQLException {
		return false;
	}

	@Override
	public String getString(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public boolean getBoolean(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public byte getByte(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public short getShort(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public int getInt(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public long getLong(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public float getFloat(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public double getDouble(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public BigDecimal getBigDecimal(int columnIndex,
			int scale) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public byte[] getBytes(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Date getDate(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Time getTime(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Timestamp getTimestamp(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public InputStream getAsciiStream(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public InputStream getUnicodeStream(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public InputStream getBinaryStream(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public String getString(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public boolean getBoolean(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public byte getByte(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public short getShort(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public int getInt(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public long getLong(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public float getFloat(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public double getDouble(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public BigDecimal getBigDecimal(String columnLabel,
			int scale) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public byte[] getBytes(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Date getDate(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Time getTime(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Timestamp getTimestamp(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public InputStream getAsciiStream(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public InputStream getUnicodeStream(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public InputStream getBinaryStream(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public SQLWarning getWarnings() throws SQLException {
		return null;
	}

	@Override
	public void clearWarnings() throws SQLException {
		// ignore it
	}

	@Override
	public String getCursorName() throws SQLException {
		return "";
	}

	@Override
	public ResultSetMetaData getMetaData() throws SQLException {
		return new NullResultSetMetaData(this);
	}

	@Override
	public Object getObject(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Object getObject(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public int findColumn(String columnLabel) throws SQLException {
		int c = columns.indexOf(columnLabel);

		if(c < 0) {
			throw ErrorBundle.getDefault(20003);
		}
		return c + 1;
	}

	@Override
	public Reader getCharacterStream(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Reader getCharacterStream(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public BigDecimal getBigDecimal(String columnLabel) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public boolean isBeforeFirst() throws SQLException {
		return false;
	}

	@Override
	public boolean isAfterLast() throws SQLException {
		return true;
	}

	@Override
	public boolean isFirst() throws SQLException {
		return false;
	}

	@Override
	public boolean isLast() throws SQLException {
		return false;
	}

	@Override
	public void beforeFirst() throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public void afterLast() throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public boolean first() throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public boolean last() throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public int getRow() throws SQLException {
		return 0;
	}

	@Override
	public boolean absolute(int row) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public boolean relative(int rows) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public boolean previous() throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public void setFetchDirection(int direction) throws SQLException {
		// ignore it
	}

	@Override
	public int getFetchDirection() throws SQLException {
		return FETCH_UNKNOWN;
	}

	@Override
	public void setFetchSize(int rows) throws SQLException {
		// ignore it
	}

	@Override
	public int getFetchSize() throws SQLException {
		return 0;
	}

	@Override
	public int getType() throws SQLException {
		return TYPE_FORWARD_ONLY;
	}

	@Override
	public int getConcurrency() throws SQLException {
		return CONCUR_READ_ONLY;
	}

	@Override
	public Statement getStatement() throws SQLException {
		return null;
	}

	@Override
	public Object getObject(int columnIndex,
			Map<String, Class<?>> map) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Ref getRef(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Blob getBlob(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Clob getClob(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Array getArray(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Object getObject(String columnLabel,
			Map<String, Class<?>> map) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Ref getRef(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Blob getBlob(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Clob getClob(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Array getArray(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Date getDate(int columnIndex, Calendar cal) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Date getDate(String columnLabel,
			Calendar cal) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Time getTime(int columnIndex,
			Calendar cal) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Time getTime(String columnLabel,
			Calendar cal) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Timestamp getTimestamp(int columnIndex,
			Calendar cal) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public Timestamp getTimestamp(String columnLabel,
			Calendar cal) throws SQLException {
		throw ErrorBundle.getDefault(20003);
	}

	@Override
	public URL getURL(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public URL getURL(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public RowId getRowId(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public RowId getRowId(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public int getHoldability() throws SQLException {
		return HOLD_CURSORS_OVER_COMMIT;
	}

	@Override
	public boolean isClosed() throws SQLException {
		return false;
	}

	@Override
	public NClob getNClob(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public NClob getNClob(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public SQLXML getSQLXML(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public SQLXML getSQLXML(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public String getNString(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public String getNString(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Reader getNCharacterStream(int columnIndex) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public Reader getNCharacterStream(String columnLabel) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

}
