package net.morilib.db.jdbc;

import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;

import net.morilib.db.misc.ErrorBundle;
import net.morilib.db.relations.NamedRelation;
import net.morilib.db.sqlcs.ddl.SqlColumnAttribute;
import net.morilib.db.sqlcs.ddl.SqlColumnDefinition;

public class RelationsResultSetMetaData implements ResultSetMetaData {

	RelationsResultSet set;

	//
	RelationsResultSetMetaData(RelationsResultSet s) {
		set = s;
	}

	@Override
	public <T> T unwrap(Class<T> iface) throws SQLException {
		throw new SQLFeatureNotSupportedException();
	}

	@Override
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return false;
	}

	@Override
	public int getColumnCount() throws SQLException {
		return set.rel.getColumnNames().size();
	}

	@Override
	public boolean isAutoIncrement(int column) throws SQLException {
		return false;
	}

	@Override
	public boolean isCaseSensitive(int column) throws SQLException {
		return false;
	}

	@Override
	public boolean isSearchable(int column) throws SQLException {
		return true;
	}

	@Override
	public boolean isCurrency(int column) throws SQLException {
		return false;
	}

	@Override
	public int isNullable(int column) throws SQLException {
		boolean b;

		b = set.rel.getColumnNames().get(column - 1)
				.getAttributes().contains(SqlColumnAttribute.NOT_NULL);
		b = b || set.rel.getColumnNames().get(column - 1)
				.getAttributes().contains(SqlColumnAttribute.PRIMARY_KEY);
		return b ? columnNoNulls : columnNullable;
	}

	@Override
	public boolean isSigned(int column) throws SQLException {
		return true;
	}

	private SqlColumnDefinition getdef(int c) throws SQLException {
		SqlColumnDefinition d;

		if(c < 1 || c > set.rel.getColumnNames().size()) {
			throw ErrorBundle.getDefault(10045, c);
		}
		d = set.rel.getColumnNames().get(c - 1);
		return d;
	}

	@Override
	public int getColumnDisplaySize(int column) throws SQLException {
		return getdef(column).getType().getLength();
	}

	@Override
	public String getColumnLabel(int column) throws SQLException {
		return getdef(column).getName();
	}

	@Override
	public String getColumnName(int column) throws SQLException {
		return getdef(column).getName();
	}

	@Override
	public String getSchemaName(int column) throws SQLException {
		return "";
	}

	@Override
	public int getPrecision(int column) throws SQLException {
		return getdef(column).getType().getLength();
	}

	@Override
	public int getScale(int column) throws SQLException {
		return getdef(column).getType().getScale();
	}

	@Override
	public String getTableName(int column) throws SQLException {
		if(set.rel instanceof NamedRelation) {
			return ((NamedRelation)set.rel).getName();
		} else {
			return "";
		}
	}

	@Override
	public String getCatalogName(int column) throws SQLException {
		return "";
	}

	@Override
	public int getColumnType(int column) throws SQLException {
		return getdef(column).getType().getSqlType();
	}

	@Override
	public String getColumnTypeName(int column) throws SQLException {
		return getdef(column).getType().toString();
	}

	@Override
	public boolean isReadOnly(int column) throws SQLException {
		return set.stmt.conn.schema.isReadonly().isTrue();
	}

	@Override
	public boolean isWritable(int column) throws SQLException {
		return set.stmt.conn.schema.isReadonly().not().isTrue();
	}

	@Override
	public boolean isDefinitelyWritable(
			int column) throws SQLException {
		return set.stmt.conn.schema.isReadonly().not().isTrue();
	}

	@Override
	public String getColumnClassName(int column) throws SQLException {
		return getdef(column).getType().getTypeClass().toString();
	}

}
