/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.db.relations;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.morilib.db.sqlcs.ddl.SqlColumnDefinition;

public class TableRelation extends AbstractRelation {

	//
	List<SqlColumnDefinition> columnNames;
	Collection<RelationTuple> data;

	/**
	 * 
	 * @param names
	 * @param data
	 */
	public TableRelation(List<SqlColumnDefinition> names,
			Collection<RelationTuple> data) {
		columnNames = new ArrayList<SqlColumnDefinition>(names);
		this.data = data;
	}

	public List<SqlColumnDefinition> getColumnNames() {
		return new ArrayList<SqlColumnDefinition>(columnNames);
	}

	public RelationCursor iterator() {
		final Iterator<RelationTuple> i = data.iterator();

		return new RelationCursor() {

			@Override
			public boolean hasNext() {
				return i.hasNext();
			}

			@Override
			public RelationTuple next() {
				return i.next();
			}

		};
	}

	@Override
	public SqlColumnDefinition getDefinition(String name) {
		for(SqlColumnDefinition d : columnNames) {
			if(d.getName().equals(name)) {
				return d;
			}
		}
		return null;
	}

}
